% Illustrate debugging demo1_d_dynamicEQ - next step:
% Preserving memory for the IIR standard difference equation
% a*y = b*x
% across frame boundaries by carrying over IIR DSP filter state
% from frame to frame.
%
% Author: G.Wakefield, 08OCT14, EECS452F14, University of Michigan

load('demo1_IIR_Butterworth_EQ');

% construct a 3-second long sinusoid at 1000 Hz
fs = 44100;
t = 0:1/fs:3;

insig = 0.5*sin(2*pi*1000*t);

% filter and playback non-framed version
outsigBass = filter(HdBass,insig);
outsigMidRange = filter(HdMidRange,insig);
outsigTreble = filter(HdTreble,insig);

outsig = outsigBass + outsigMidRange + outsigTreble;

% plot results
idx = 1:5000;
subplot(2,1,1)
plot(1000*t(idx),outsig(idx));
xlabel('t [msec]')
ylabel('amplitude');
title('Output from Batch-processed EQ')
v = axis;
v(2) = 1000*t(max(idx));
axis(v)
drawnow

aPBatch = audioplayer(outsig,fs);
playblocking(aPBatch); 

% break input up into frameSz chunks, process, and concatenate
outsigBass = [];
outsigMidRange = [];
outsigTreble = [];

frameSz = 1024;
iStart = 1;
iStop = frameSz;
nSig = length(insig);

gBass = 1;
gMidRange = 1;
gTreble = 1;

set([HdBass HdMidRange HdTreble],'PersistentMemory',1);  % do not clear filter state each time you filter

while iStop <= nSig
    idx = iStart:iStop;
    outsigBass = [outsigBass gBass*filter(HdBass,insig(idx))];
    outsigMidRange = [outsigMidRange gMidRange*filter(HdMidRange,insig(idx))];
    outsigTreble = [outsigTreble gTreble*filter(HdTreble,insig(idx))];
    iStart = iStart + frameSz;
    iStop = iStop + frameSz;
end;

outsig = outsigBass + outsigMidRange + outsigTreble;

% listen to and look at the result
subplot(2,1,2)
idx = 1:5000;
plot(1000*t(idx),outsig(idx));
xlabel('t [msec]')
ylabel('amplitude');
title('Output from RT-processed EQ')
v = axis;
v(2) = 1000*t(max(idx));
axis(v)
drawnow

aPFrame = audioplayer(outsig,fs);
playblocking(aPFrame);

% Q: Sounds good but why is there an onset difference between the "batch"-
% and "frame"-processed outputs?
% A: Look at the code to determine whether the states of each filter are
% the same when they are first asserted.
