% Incorporating solution back into the NAIVE demo1_d code and using this
% approach to implement a (simple) time-varying linear system, which would
% otherwise be challenging in a "batch" mode of processing.
%
% Author: G.Wakefield, 08OCT14, EECS452F14, University of Michigan

load('demo1_IIR_Butterworth_EQ');


% break input up into frameSz chunks, process, and concatenate
info = audioinfo('SuperstitionSnippet.wav');

frameSz = 1024;
iStart = 1;
iStop = frameSz;
nSig = info.TotalSamples;

% specify time-varying linear system as a piecewise-LTI components with
% slowly varying gain over time

nFrame = floor(nSig/frameSz);
for iFrame = 1:nFrame
    gainBass(iFrame) = 0-60*iFrame/nFrame;
    gainMidRange(iFrame) = -10;
    gainTreble(iFrame) = -60+60*iFrame/nFrame;
end;

% visualize amplitude variations
plot(gainBass,'r');
hold on
plot(gainMidRange,'g');
plot(gainTreble,'b');
legend('Bass','MidRange','Treble');
xlabel('frame index');
ylabel('filter gain [dB]');
hold off
drawnow

% execute frame-based processing

outsigBass = [];
outsigMidRange = [];
outsigTreble = [];

gBass = 1;
gMidRange = 1;
gTreble = 1;

set([HdBass HdMidRange HdTreble],'PersistentMemory',1);

iC = 1;
while iStop <= nSig
    [insig, fs] = audioread('SuperstitionSnippet.wav',[iStart iStop]);
    
    aBass = 10^(gainBass(iC)/20);
    aMidRange = 10^(gainMidRange(iC)/20);
    aTreble = 10^(gainTreble(iC)/20);
    
    outsigBass = [outsigBass ; aBass*filter(HdBass,insig)];
    outsigMidRange = [outsigMidRange ; aMidRange*filter(HdMidRange,insig)];
    outsigTreble = [outsigTreble ; aTreble*filter(HdTreble,insig)];
    
    iStart = iStart + frameSz;
    iStop = iStop + frameSz;
    iC = iC + 1;
end;

outsig = outsigBass + outsigMidRange + outsigTreble;

% listen to the result
aP = audioplayer(outsig,fs);
playblocking(aP);
