% Combining dsp I/O and filtering - implement demo1_g_debug_dynamicEQ in
% real-time.
%
% Author: G.Wakefield, 08OCT14, EECS452F14, University of Michigan
%

% set up audio I/O
inAudioDev = dsp.AudioFileReader('SuperstitionSnippet.wav');
inInfo = audioinfo('SuperstitionSnippet.wav');

outAudioDev = dsp.AudioPlayer('SampleRate',inAudioDev.SampleRate);

frameSz = inAudioDev.SamplesPerFrame;
nInput = inInfo.TotalSamples;

% load equalization filters
load('demo1_IIR_Butterworth_EQ');

% create equalizer amplitude contours
nFrame = ceil(nInput/frameSz);
for iFrame = 1:nFrame
    gainBass(iFrame) = 0-60*iFrame/nFrame;
    gainMidRange(iFrame) = -10;
    gainTreble(iFrame) = -60+60*iFrame/nFrame;
end;

% prep digital filter objects for realtime execution
set([HdBass HdMidRange HdTreble],'PersistentMemory',1);

% do the job - input, process, and playback on the fly
iC = 1;
while ~isDone(inAudioDev)
    insig = step(inAudioDev);
    
    aBass = 10^(gainBass(iC)/20);
    aMidRange = 10^(gainMidRange(iC)/20);
    aTreble = 10^(gainTreble(iC)/20);
    
    outsig = aBass*filter(HdBass,insig);
    outsig = outsig + aMidRange*filter(HdMidRange,insig);
    outsig = outsig + aTreble*filter(HdTreble,insig);

    iC = iC + 1;
    
    step(outAudioDev,outsig);
end

release(inAudioDev);
release(outAudioDev);

