% Expand functionality of demo3_a_prep: 
% Modify the gain of each subsystem over frames.
% 
% Author: G.Wakefield, 08OCT14, EECS452F14, University of Michigan
% 

inAudioDev = dsp.AudioFileReader('SuperstitionSnippet.wav');
inInfo = audioinfo('SuperstitionSnippet.wav');

outAudioDev = dsp.AudioPlayer('SampleRate',inAudioDev.SampleRate);

load('demo1_IIR_Butterworth_EQ');

% create EqFilter objects as the cascade of a filter and a multiplier
EqBass = cascade(HdBass,dfilt.scalar(1));
EqMidRange = cascade(HdMidRange,dfilt.scalar(1));
EqTreble = cascade(HdTreble,dfilt.scalar(1));

% create Eq object as the parallel combination of the EqFilter objects
Eq = parallel(EqBass,EqMidRange,EqTreble);

% prep Eq for realtime execution
set(Eq,'PersistentMemory',1);

% % prep digital filter objects for realtime execution
% set([HdBass HdMidRange HdTreble],'PersistentMemory',1);

frameSz = inAudioDev.SamplesPerFrame;
nInput = inInfo.TotalSamples;

% create equalizer amplitude contours
nFrame = ceil(nInput/frameSz);
for iFrame = 1:nFrame
    gainBass(iFrame) = 0-60*iFrame/nFrame;
    gainMidRange(iFrame) = -10;
    gainTreble(iFrame) = -60+60*iFrame/nFrame;
end;

iC = 1;
while ~isDone(inAudioDev)
    insig = step(inAudioDev);
    
    aBass = 10^(gainBass(iC)/20);
    aMidRange = 10^(gainMidRange(iC)/20);
    aTreble = 10^(gainTreble(iC)/20);
    
    % change gains on the fly using Stage attribute of parallel and cascade
    % objects
    Eq.Stage(1).Stage(2).Gain = aBass;
    Eq.Stage(2).Stage(2).Gain = aMidRange;
    Eq.Stage(3).Stage(2).Gain = aTreble;
    
    outsig = filter(Eq,insig);

    iC = iC + 1;
    
    step(outAudioDev,outsig);
end

release(inAudioDev);
release(outAudioDev);

