% Extend functionality of demo3_a_prep: design dsp filters on the fly
% rather than presetting and loading from disk.
% Illustrate use of fdesign and design
%
% Author: G.Wakefield, 08OCT14, EECS452F14, University of Michigan
%

inAudioDev = dsp.AudioFileReader('SuperstitionSnippet.wav');
inInfo = audioinfo('SuperstitionSnippet.wav');

outAudioDev = dsp.AudioPlayer('SampleRate',inAudioDev.SampleRate);

% create lowpass, bandpass and highpass filters
% load('demo1_IIR_Butterworth_EQ');    % old code
dBass = fdesign.lowpass('N,F3dB',6,320/22050);
HdBass = design(dBass,'butter');

dMidRange = fdesign.bandpass('N,F3dB1,F3dB2',6,320/22050,1280/22050);
HdMidRange = design(dMidRange,'butter');

dTreble = fdesign.highpass('N,F3dB',6,1280/22050);
HdTreble = design(dTreble,'butter');

% create EqFilter objects as the cascade of a filter and a multiplier
EqBass = cascade(HdBass,dfilt.scalar(1));
EqMidRange = cascade(HdMidRange,dfilt.scalar(1));
EqTreble = cascade(HdTreble,dfilt.scalar(1));

% create Eq object as the parallel combination of the EqFilter objects
Eq = parallel(EqBass,EqMidRange,EqTreble);

% prep Eq for realtime execution
set(Eq,'PersistentMemory',1);

% % prep digital filter objects for realtime execution
% set([HdBass HdMidRange HdTreble],'PersistentMemory',1);

frameSz = inAudioDev.SamplesPerFrame;
nInput = inInfo.TotalSamples;

while ~isDone(inAudioDev)
    insig = step(inAudioDev);
    
    outsig = filter(Eq,insig);
    
    step(outAudioDev,outsig);
end

release(inAudioDev);
release(outAudioDev);

