% Extend functionality of demo3_a_prep: design dsp filters on the fly
% rather than presetting and loading from disk.
% Illustrate use of fdesign and design
%
% Author: G.Wakefield, 08OCT14, EECS452F14, University of Michigan
%

inAudioDev = dsp.AudioFileReader('SuperstitionSnippet.wav');
inInfo = audioinfo('SuperstitionSnippet.wav');

outAudioDev = dsp.AudioPlayer('SampleRate',inAudioDev.SampleRate);

% create filters
dBass = fdesign.lowpass('N,F3dB',6,320/22050);
HdBass = design(dBass,'butter');

dMidRange = fdesign.bandpass('N,F3dB1,F3dB2',6,320/22050,1280/22050);
HdMidRange = design(dMidRange,'butter');

dTreble = fdesign.highpass('N,F3dB',6,1280/22050);
HdTreble = design(dTreble,'butter');

% create EqFilter objects as the cascade of a filter and a multiplier
EqBass = cascade(HdBass,dfilt.scalar(1));
EqMidRange = cascade(HdMidRange,dfilt.scalar(1));
EqTreble = cascade(HdTreble,dfilt.scalar(1));

% create Eq object as the parallel combination of the EqFilter objects
Eq = parallel(EqBass,EqMidRange,EqTreble);

% prep Eq for realtime execution
set(Eq,'PersistentMemory',1);

frameSz = inAudioDev.SamplesPerFrame;
nInput = inInfo.TotalSamples;

% create equalizer amplitude contours
nFrame = ceil(nInput/frameSz);
for iFrame = 1:nFrame
    gainBass(iFrame) = 0-60*iFrame/nFrame;
    gainMidRange(iFrame) = 0;
    gainTreble(iFrame) = -60+60*iFrame/nFrame;
    MidRangeFreq(iFrame) = (8000-100)*iFrame/nFrame+100;    % NEW
end;

MidRangeQ = 8;

iC = 1;
while ~isDone(inAudioDev)
    insig = step(inAudioDev);
    
    aBass = 10^(gainBass(iC)/20);
    aMidRange = 10^(gainMidRange(iC)/20);
    aTreble = 10^(gainTreble(iC)/20);
    
    % change gains on the fly using Stage attribute of parallel and cascade
    % objects
    Eq.Stage(1).Stage(2).Gain = aBass;
    Eq.Stage(2).Stage(2).Gain = aMidRange;
    Eq.Stage(3).Stage(2).Gain = aTreble;
    
    % change MidRange Stage on the fly          % NEW
    MidRangeBW = MidRangeFreq(iC)/MidRangeQ;
    F3dB1 = MidRangeFreq(iC)-MidRangeBW/2;
    F3dB2 = MidRangeFreq(iC)+MidRangeBW/2;
    dMidRange = fdesign.bandpass('N,F3dB1,F3dB2',6,F3dB1/22050,F3dB2/22050);
    HdMidRange = design(dMidRange,'butter');
    Eq.Stage(2).Stage(1).sosMatrix = HdMidRange.sosMatrix;
    Eq.Stage(2).Stage(1).ScaleValues = HdMidRange.ScaleValues;
    
    outsig = filter(Eq,insig);

%     outsig = aBass*filter(HdBass,insig);
%     outsig = outsig + aMidRange*filter(HdMidRange,insig);
%     outsig = outsig + aTreble*filter(HdTreble,insig);

    iC = iC + 1;
    
    step(outAudioDev,outsig);
end

release(inAudioDev);
release(outAudioDev);

