function varargout = demo4_latency(varargin)
% Introduce programmable QueueDuration and BufferSize controls for
% minimizing latency. Recode the Pause and Stop states at the UIControl
% level using radiobuttons, i.e., eliminate Callback for Pause and Stop
% pushbuttons.
% (Hint: You are likely to find that a QueueDuration of 0.1 sec and a buffer size of 1024
% yields a reasonable improvement in responsiveness.]
%
% Author: G.Wakefield, 08OCT14, EECS452F14, University of Michigan
%
% DEMO4_LATENCY MATLAB code for demo4_latency.fig
%      DEMO4_LATENCY, by itself, creates a new DEMO4_LATENCY or raises the existing
%      singleton*.
%
%      H = DEMO4_LATENCY returns the handle to a new DEMO4_LATENCY or the handle to
%      the existing singleton*.
%
%      DEMO4_LATENCY('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in DEMO4_LATENCY.M with the given input arguments.
%
%      DEMO4_LATENCY('Property','Value',...) creates a new DEMO4_LATENCY or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before demo4_latency_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to demo4_latency_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help demo4_latency

% Last Modified by GUIDE v2.5 08-Oct-2014 12:34:48

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @demo4_latency_OpeningFcn, ...
    'gui_OutputFcn',  @demo4_latency_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before demo4_latency is made visible.
function demo4_latency_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to demo4_latency (see VARARGIN)

% Choose default command line output for demo4_latency
handles.output = hObject;

set([handles.pushPlay handles.pushPause handles.pushStop],'Enable','off');
handles.pauseState = false;
handles.stopState = false;

handles.eqSliders = [handles.sliderBass handles.sliderMidRange handles.sliderTreble];
set(handles.eqSliders,'Min',-40,'Max',0,'Value',-8);

load('demo1_IIR_Butterworth_EQ');
set([HdBass HdMidRange HdTreble],'PersistentMemory',1);
handles.HdBass = HdBass;
handles.HdMidRange = HdMidRange;
handles.HdTreble = HdTreble;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes demo4_latency wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = demo4_latency_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in pushLoad.
function pushLoad_Callback(hObject, eventdata, handles)
% hObject    handle to pushLoad (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

[fName pName] = uigetfile({'*.wav; *.m4a; *.avi; *.mp3'});
handles.audFileName = [pName fName];

handles.inAudioDev = dsp.AudioFileReader(handles.audFileName);
handles.outAudioDev = dsp.AudioPlayer('SampleRate',handles.inAudioDev.SampleRate);

% set QueueDuration [sec]
set(handles.outAudioDev,'QueueDuration',str2num(get(handles.editQDur,'String')));

% set BufferSize [samples]
set(handles.outAudioDev,'BufferSizeSource','Property');
set(handles.outAudioDev,'BufferSize',str2num(get(handles.editBufferSz,'String')));

set(hObject,'String','Thank you','Enable','off');
set(handles.pushPlay,'Enable','on');

handles.pauseState = false;
handles.stopState = false;

guidata(hObject,handles);

% --- Executes on button press in pushPlay.
function pushPlay_Callback(hObject, eventdata, handles)
% hObject    handle to pushPlay (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% reuse the demo1_h code
% to improve latency, remove all possible initializations 

set(hObject,'String','Playing...','Enable','off');
% set([handles.pushPause handles.pushStop],'Enable','on');
drawnow

% move initialization to pushLoad_Callback and elevate xDev to handles
% fields
% inAudioDev = dsp.AudioFileReader(handles.audFileName);
% outAudioDev = dsp.AudioPlayer('SampleRate',inAudioDev.SampleRate);

% move load to OpeningFcn and elevate HdX to handles fields
% load('demo1_IIR_Butterworth_EQ');
% set([HdBass HdMidRange HdTreble],'PersistentMemory',1);

while ~isDone(handles.inAudioDev) && ~get(handles.radioStop,'Value')
    while get(handles.radioPause,'Value')
        drawnow
    end
    
    sig = step(handles.inAudioDev);
    
    gainBass = get(handles.sliderBass,'Value');
    gainMidRange = get(handles.sliderMidRange,'Value');
    gainTreble = get(handles.sliderTreble,'Value');
    
    aBass = 10^(gainBass/20);
    aMidRange = 10^(gainMidRange/20);
    aTreble = 10^(gainTreble/20);
    
    outsig = aBass*filter(handles.HdBass,sig);
    outsig = outsig + aMidRange*filter(handles.HdMidRange,sig);
    outsig = outsig + aTreble*filter(handles.HdTreble,sig);
    
    step(handles.outAudioDev,outsig);
    drawnow
end

release(handles.inAudioDev);
release(handles.outAudioDev);

set(hObject,'String','Play Me');
set([handles.pushPlay handles.pushPause handles.pushStop],'Enable','off');
set(handles.pushLoad,'String','Load Me','Enable','on');
set(handles.pushPause,'String','Pause');
set([handles.radioPause handles.radioStop],'Value',0);
drawnow


% --- Executes on button press in pushPause.
function pushPause_Callback(hObject, eventdata, handles)
% hObject    handle to pushPause (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if ~handles.pauseState
    handles.pauseState = true;
    set(handles.pushPause,'String','Resume');
    guidata(hObject,handles);
else
    handles.pauseState = false;
    set(handles.pushPause,'String','Pause');
    guidata(hObject,handles);
end;

% --- Executes on button press in pushStop.
function pushStop_Callback(hObject, eventdata, handles)
% hObject    handle to pushStop (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.stopState = 1;
guidata(hObject,handles);


% --- Executes on slider movement.
function sliderBass_Callback(hObject, eventdata, handles)
% hObject    handle to sliderBass (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider


% --- Executes during object creation, after setting all properties.
function sliderBass_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sliderBass (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes on slider movement.
function sliderMidRange_Callback(hObject, eventdata, handles)
% hObject    handle to sliderMidRange (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider


% --- Executes during object creation, after setting all properties.
function sliderMidRange_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sliderMidRange (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


% --- Executes on slider movement.
function sliderTreble_Callback(hObject, eventdata, handles)
% hObject    handle to sliderTreble (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider


% --- Executes during object creation, after setting all properties.
function sliderTreble_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sliderTreble (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end



function editQDur_Callback(hObject, eventdata, handles)
% hObject    handle to editQDur (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of editQDur as text
%        str2double(get(hObject,'String')) returns contents of editQDur as a double


% --- Executes during object creation, after setting all properties.
function editQDur_CreateFcn(hObject, eventdata, handles)
% hObject    handle to editQDur (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function editBufferSz_Callback(hObject, eventdata, handles)
% hObject    handle to editBufferSz (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of editBufferSz as text
%        str2double(get(hObject,'String')) returns contents of editBufferSz as a double


% --- Executes during object creation, after setting all properties.
function editBufferSz_CreateFcn(hObject, eventdata, handles)
% hObject    handle to editBufferSz (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in radioPause.
function radioPause_Callback(hObject, eventdata, handles)
% hObject    handle to radioPause (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radioPause

% --- Executes on button press in radioStop.
function radioStop_Callback(hObject, eventdata, handles)
% hObject    handle to radioStop (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of radioStop
